@@ -53,7 +53,7 @@ class Action(object):
         return True
 
     @staticmethod
-    def execute(act_id, data, action_type=None, context=None):
+    def execute(act_id, data, action_type=None, context=None, keyword=False):
         # Must be executed synchronously to avoid double execution
         # on double click.
         if not action_type:
@@ -63,17 +63,19 @@ class Action(object):
         action, = RPCExecute('model', action_type, 'search_read',
             [('action', '=', act_id)], 0, 1, None, None,
             context=context)
+        if keyword:
+            keywords = {
+                'ir.action.report': 'form_report',
+                'ir.action.wizard': 'form_action',
+                'ir.action.act_window': 'form_relate',
+                }
+            action.setdefault('keyword', keywords.get(action_type, ''))
         Action._exec_action(action, data, context=context)
 
     @staticmethod
     def _exec_action(action, data=None, context=None):
         if context is None:
             context = {}
-        else:
-            context = context.copy()
-        if 'date_format' not in context:
-            context['date_format'] = rpc.CONTEXT.get(
-                'locale', {}).get('date', '%x')
         if data is None:
             data = {}
         else:
@@ -81,9 +83,20 @@ class Action(object):
         if 'type' not in (action or {}):
             return
 
+        def add_name_suffix(name):
+            if not data.get('ids') or not data.get('model'):
+                return name
+            max_records = 5
+            rec_names = RPCExecute('model', data['model'],
+                'read', data['ids'][:max_records], ['rec_name'])
+            name_suffix = _(', ').join([x['rec_name'] for x in rec_names])
+            if len(data['ids']) > max_records:
+                name_suffix += _(u',\u2026')
+            return _('%s (%s)') % (name, name_suffix)
+
         data['action_id'] = action['id']
         if action['type'] == 'ir.action.act_window':
-            view_ids = False
+            view_ids = []
             view_mode = None
             if action.get('views', []):
                 view_ids = [x[0] for x in action['views']]
@@ -111,27 +124,38 @@ class Action(object):
             domain = decoder.decode(action['pyson_domain'])
             order = decoder.decode(action['pyson_order'])
             search_value = decoder.decode(action['pyson_search_value'] or '[]')
-            tab_domain = [(n, decoder.decode(d)) for n, d in action['domains']]
+            tab_domain = [(n, decoder.decode(d), c)
+                for n, d, c in action['domains']]
 
-            name = False
-            if action.get('window_name', True):
-                name = action.get('name', False)
+            name = action.get('name', '')
+            if action.get('keyword', '') == 'form_relate':
+                name = add_name_suffix(name)
 
             res_model = action.get('res_model', data.get('res_model'))
             res_id = action.get('res_id', data.get('res_id'))
 
-            Window.create(view_ids, res_model, res_id, domain,
-                    action_ctx, order, view_mode, name=name,
-                    limit=action.get('limit'),
-                    search_value=search_value,
-                    icon=(action.get('icon.rec_name') or ''),
-                    tab_domain=tab_domain,
-                    context_model=action['context_model'])
+            Window.create(res_model,
+                view_ids=view_ids,
+                res_id=res_id,
+                domain=domain,
+                context=action_ctx,
+                order=order,
+                mode=view_mode,
+                name=name,
+                limit=action.get('limit'),
+                search_value=search_value,
+                icon=(action.get('icon.rec_name') or ''),
+                tab_domain=tab_domain,
+                context_model=action['context_model'],
+                context_domain=action['context_domain'])
         elif action['type'] == 'ir.action.wizard':
+            name = action.get('name', '')
+            if action.get('keyword', 'form_action') == 'form_action':
+                name = add_name_suffix(name)
             Window.create_wizard(action['wiz_name'], data,
                 direct_print=action.get('direct_print', False),
                 email_print=action.get('email_print', False),
-                email=action.get('email'), name=action.get('name', False),
+                email=action.get('email'), name=name,
                 context=context, icon=(action.get('icon.rec_name') or ''),
                 window=action.get('window', False))
 
@@ -166,7 +190,7 @@ class Action(object):
             Action._exec_action(action, data, context=context)
             return (name, action)
         elif not len(keyact) and warning:
-            message(_('No action defined!'))
+            message(_('No action defined.'))
         return False
 
     @staticmethod
