@@ -41,6 +41,9 @@ if os.environ.get('GTK_VERSION', '2').startswith('3'):
     Gtk.PROGRESS_BOTTOM_TO_TOP = (Gtk.Orientation.VERTICAL, True)
     Gtk.PROGRESS_TOP_TO_BOTTOM = (Gtk.Orientation.VERTICAL, False)
 
+    Gtk.CLIPBOARD_PRIMARY = Gdk.Atom.intern('PRIMARY', True)
+    Gtk.CLIPBOARD_CLIPBOARD = Gdk.Atom.intern('CLIPBOARD', True)
+
     orig_tree_view_column_set_cell_data_func = (
         Gtk.TreeViewColumn.set_cell_data_func)
 
@@ -104,24 +107,6 @@ if os.environ.get('GTK_VERSION', '2').startswith('3'):
     Pango.SCALE_LARGE = 1.2
     Pango.SCALE_X_LARGE = 1.2 * 1.2
     Pango.SCALE_XX_LARGE = 1.2 * 1.2 * 1.2
-
-    style_provider = Gtk.CssProvider()
-    css = """
-    #editable {
-        background-color: @theme_base_color;
-        color: @theme_text_color;
-    }
-    #readonly {
-        background-color: @insensitive_bg_color;
-        color: @insensitive_fg_color;
-    }
-    """
-    style_provider.load_from_data(css)
-    Gtk.StyleContext.add_provider_for_screen(
-        Gdk.Screen.get_default(),
-        style_provider,
-        Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
-
 else:
     import pygtk
     pygtk.require('2.0')
@@ -132,6 +117,9 @@ if not hasattr(gtk, 'TreePath'):
 if not hasattr(gtk, 'TargetEntry'):
     gtk.TargetEntry = lambda *a: a
     gtk.TargetEntry.new = lambda *a: a
+if not hasattr(gtk, 'CLIPBOARD_PRIMARY'):
+    gtk.CLIPBOARD_PRIMARY = 'PRIMARY'
+    gtk.CLIPBOARD_CLIPBOARD = 'CLIPBOARD'
 
 import gobject
 try:
@@ -218,8 +206,6 @@ class TrytonClient(object):
 
         sys.excepthook = excepthook
 
-        if CONFIG['tip.autostart']:
-            main.sig_tips()
         main.sig_login()
 
         if sys.platform == 'win32':
@@ -233,7 +219,7 @@ class TrytonClient(object):
             if sys.platform == 'win32':
                 while not self.quit_client.isSet():
                     with gtk.gdk.lock:
-                            gtk.main_iteration(True)
+                            gtk.main_iteration()
             else:
                 gtk.main()
         except KeyboardInterrupt:
