@@ -32,6 +32,7 @@ class Record(SignalEvent):
         self._timestamp = None
         self.attachment_count = -1
         self.unread_note = -1
+        self.button_clicks = {}
         self.next = {}  # Used in Group list
         self.value = {}
         self.autocompletion = {}
@@ -71,7 +72,7 @@ class Record(SignalEvent):
                 fnames.append('rec_name')
             fnames.append('_timestamp')
 
-            record_context = self.context_get()
+            record_context = self.get_context()
             if loading == 'eager':
                 limit = int(CONFIG['client.limit'] / len(fnames))
 
@@ -83,7 +84,7 @@ class Record(SignalEvent):
                         and record.id not in id2record
                         and ((record.group == self.group)
                             # Don't compute context for same group
-                            or (record.context_get() == record_context)))
+                            or (record.get_context() == record_context)))
 
                 if self.parent and self.parent.model_name == self.model_name:
                     group = sum(self.parent.group.children, [])
@@ -196,6 +197,17 @@ class Record(SignalEvent):
         path.reverse()
         return tuple(path)
 
+    def get_index_path(self, group=None):
+        path = []
+        record = self
+        while record:
+            path.append(record.group.index(record))
+            if record.group is group:
+                break
+            record = record.parent
+        path.reverse()
+        return tuple(path)
+
     def get_removed(self):
         if self.group is not None:
             return self in self.group.record_removed
@@ -239,7 +251,8 @@ class Record(SignalEvent):
     def get(self):
         value = {}
         for name, field in self.group.fields.iteritems():
-            if field.attrs.get('readonly'):
+            if (field.attrs.get('readonly')
+                    and not isinstance(field, fields.O2MField)):
                 continue
             if field.name not in self.modified_fields and self.id >= 0:
                 continue
@@ -272,6 +285,7 @@ class Record(SignalEvent):
         self._loaded.clear()
         self.modified_fields.clear()
         self._timestamp = None
+        self.button_clicks.clear()
 
     def get_timestamp(self):
         result = {self.model_name + ',' + str(self.id): self._timestamp}
@@ -286,7 +300,7 @@ class Record(SignalEvent):
         values = self._get_on_change_args(self.modified_fields)
         try:
             RPCExecute('model', self.model_name, 'pre_validate', values,
-                context=self.context_get())
+                context=self.get_context())
         except RPCException:
             return False
         return True
@@ -297,7 +311,7 @@ class Record(SignalEvent):
                 value = self.get()
                 try:
                     res, = RPCExecute('model', self.model_name, 'create',
-                        [value], context=self.context_get())
+                        [value], context=self.get_context())
                 except RPCException:
                     return False
                 old_id = self.id
@@ -306,7 +320,7 @@ class Record(SignalEvent):
             elif self.modified:
                 value = self.get()
                 if value:
-                    context = self.context_get()
+                    context = self.get_context()
                     context = context.copy()
                     context['_timestamp'] = self.get_timestamp()
                     try:
@@ -350,11 +364,13 @@ class Record(SignalEvent):
             root_group.reload(reload_ids)
         return True
 
-    def default_get(self):
+    def default_get(self, rec_name=None):
         if len(self.group.fields):
+            context = self.get_context()
+            context.setdefault('default_rec_name', rec_name)
             try:
                 vals = RPCExecute('model', self.model_name, 'default_get',
-                    self.group.fields.keys(), context=self.context_get())
+                    self.group.fields.keys(), context=context)
             except RPCException:
                 return
             if (self.parent
@@ -367,16 +383,12 @@ class Record(SignalEvent):
                         == self.group.parent.model_name):
                     vals[self.parent_name] = self.parent.id
             self.set_default(vals)
-        for fieldname, fieldinfo in self.group.fields.iteritems():
-            if not fieldinfo.attrs.get('autocomplete'):
-                continue
-            self.do_autocomplete(fieldname)
         return vals
 
     def rec_name(self):
         try:
             return RPCExecute('model', self.model_name, 'read', [self.id],
-                ['rec_name'], context=self.context_get())[0]['rec_name']
+                ['rec_name'], context=self.get_context())[0]['rec_name']
         except RPCException:
             return ''
 
@@ -389,7 +401,7 @@ class Record(SignalEvent):
         for field_name, field in self.group.fields.iteritems():
             if fields is not None and field_name not in fields:
                 continue
-            if field.get_state_attrs(self).get('readonly', False):
+            if field.attrs.get('readonly'):
                 continue
             if field_name == self.group.exclude_field:
                 continue
@@ -407,7 +419,7 @@ class Record(SignalEvent):
 
     invalid_fields = property(_get_invalid_fields)
 
-    def context_get(self):
+    def get_context(self):
         return self.group.context
 
     def set_default(self, val, signal=True, validate=True):
@@ -434,8 +446,9 @@ class Record(SignalEvent):
         if signal:
             self.signal('record-changed')
 
-    def set(self, val, signal=True):
+    def set(self, val, signal=True, validate=True):
         later = {}
+        fieldnames = []
         for fieldname, value in val.iteritems():
             if fieldname == '_timestamp':
                 # Always keep the older timestamp
@@ -458,9 +471,12 @@ class Record(SignalEvent):
                     del self.value[field_rec_name]
             self.group.fields[fieldname].set(self, value)
             self._loaded.add(fieldname)
+            fieldnames.append(fieldname)
         for fieldname, value in later.iteritems():
             self.group.fields[fieldname].set(self, value)
             self._loaded.add(fieldname)
+        if validate:
+            self.validate(fieldnames, softvalidation=True)
         if signal:
             self.signal('record-changed')
 
@@ -492,7 +508,7 @@ class Record(SignalEvent):
             return expr
         ctx = rpc.CONTEXT.copy()
         ctx['context'] = ctx.copy()
-        ctx['context'].update(self.context_get())
+        ctx['context'].update(self.get_context())
         ctx.update(self.get_eval())
         ctx['active_model'] = self.model_name
         ctx['active_id'] = self.id
@@ -528,7 +544,7 @@ class Record(SignalEvent):
         if values:
             try:
                 changes = RPCExecute('model', self.model_name, 'on_change',
-                    values, fieldnames, context=self.context_get())
+                    values, fieldnames, context=self.get_context())
             except RPCException:
                 return
             for change in changes:
@@ -559,7 +575,7 @@ class Record(SignalEvent):
         if fieldnames:
             try:
                 result = RPCExecute('model', self.model_name, 'on_change_with',
-                    values, list(fieldnames), context=self.context_get())
+                    values, list(fieldnames), context=self.get_context())
             except RPCException:
                 return
             self.set_on_change(result)
@@ -570,7 +586,7 @@ class Record(SignalEvent):
             try:
                 result = RPCExecute('model', self.model_name,
                     'on_change_with_' + fieldname, values,
-                    context=self.context_get())
+                    context=self.get_context())
             except RPCException:
                 return
             self.group.fields[fieldname].set_on_change(self, result)
@@ -588,7 +604,7 @@ class Record(SignalEvent):
         args = self._get_on_change_args(autocomplete)
         try:
             res = RPCExecute('model', self.model_name, 'autocomplete_' +
-                fieldname, args, context=self.context_get())
+                fieldname, args, context=self.get_context())
         except RPCException:
             # ensure res is a list
             res = []
@@ -613,7 +629,7 @@ class Record(SignalEvent):
                     'search_count', [
                         ('resource', '=',
                             '%s,%s' % (self.model_name, self.id)),
-                        ], context=self.context_get())
+                        ], context=self.get_context())
             except RPCException:
                 return 0
         return self.attachment_count
@@ -628,11 +644,25 @@ class Record(SignalEvent):
                         ('resource', '=',
                             '%s,%s' % (self.model_name, self.id)),
                         ('unread', '=', True),
-                        ], context=self.context_get())
+                        ], context=self.get_context())
             except RPCException:
                 return 0
         return self.unread_note
 
+    def get_button_clicks(self, name):
+        if self.id < 0:
+            return
+        clicks = self.button_clicks.get(name)
+        if clicks is not None:
+            return clicks
+        try:
+            clicks = RPCExecute('model', 'ir.model.button.click',
+                'get_click', self.model_name, name, self.id)
+            self.button_clicks[name] = clicks
+        except RPCException:
+            return
+        return clicks
+
     def destroy(self):
         for v in self.value.itervalues():
             if hasattr(v, 'destroy'):
