@@ -187,7 +187,8 @@ class One2Many(Widget):
             view_ids=attrs.get('view_ids', '').split(','),
             views_preload=attrs.get('views', {}),
             row_activate=self._on_activate,
-            exclude_field=attrs.get('relation_field', None))
+            exclude_field=attrs.get('relation_field', None),
+            limit=None)
         self.screen.pre_validate = bool(int(attrs.get('pre_validate', 0)))
         self.screen.signal_connect(self, 'record-message', self._sig_label)
 
@@ -325,6 +326,13 @@ class One2Many(Widget):
                 return False
         return True
 
+    def _sequence(self):
+        for view in self.screen.views:
+            if view.view_type == 'tree':
+                sequence = view.attributes.get('sequence')
+                if sequence:
+                    return sequence
+
     def _sig_new(self, *args):
         if not common.MODELACCESS[self.screen.model_name]['create']:
             return
@@ -338,13 +346,8 @@ class One2Many(Widget):
 
     def _new_single(self):
         ctx = {}
-        ctx.update(self.field.context_get(self.record))
-        sequence = None
-        for view in self.screen.views:
-            if view.view_type == 'tree':
-                sequence = view.attributes.get('sequence')
-                if sequence:
-                    break
+        ctx.update(self.field.get_context(self.record))
+        sequence = self._sequence()
 
         def update_sequence():
             if sequence:
@@ -377,7 +380,7 @@ class One2Many(Widget):
                 search_set()
 
             domain = field.domain_get(first)
-            context = field.context_get(first)
+            context = field.get_context(first)
 
             def callback(result):
                 if result:
@@ -408,6 +411,10 @@ class One2Many(Widget):
                     default_value[field + '.rec_name'] = rec_name
                 record.set_default(default_value)
 
+            sequence = self._sequence()
+            if sequence:
+                self.screen.group.set_sequence(field=sequence)
+
         search_set()
 
     def _sig_edit(self, widget=None):
@@ -451,7 +458,7 @@ class One2Many(Widget):
             return
         self.view.set_value()
         domain = self.field.domain_get(self.record)
-        context = self.field.context_get(self.record)
+        context = self.field.get_context(self.record)
         domain = [domain, self.record.expr_eval(self.attrs.get('add_remove'))]
         removed_ids = self.field.get_removed_ids(self.record)
         domain = ['OR', domain, ('id', 'in', removed_ids)]
@@ -459,9 +466,7 @@ class One2Many(Widget):
 
         self.focus_out = False
 
-        sequence = None
-        if self.screen.current_view.view_type == 'tree':
-            sequence = self.screen.current_view.attributes.get('sequence')
+        sequence = self._sequence()
 
         def callback(result):
             self.focus_out = True
@@ -517,6 +522,11 @@ class One2Many(Widget):
         if record:
             domain = field.domain_get(record)
             size_limit = record.expr_eval(self.attrs.get('size'))
+        if self._readonly:
+            if size_limit is None:
+                size_limit = len(self.screen.group)
+            else:
+                size_limit = min(size_limit, len(self.screen.group))
         if self.screen.domain != domain:
             self.screen.domain = domain
         self.screen.size_limit = size_limit
