@@ -9,7 +9,6 @@ from tryton.common.domain_parser import quote
 from tryton.common.placeholder_entry import PlaceholderEntry
 from tryton.common.treeviewcontrol import TreeViewControl
 from tryton.common.datetime_ import Date, Time, DateTime, add_operators
-from tryton.config import TRYTON_ICON
 from tryton.pyson import PYSONDecoder
 
 _ = gettext.gettext
@@ -62,7 +61,7 @@ class Times(Dates):
 
     def connect_activate(self, callback):
         for widget in self.from_.get_children() + self.to.get_children():
-            widget.child.connect('activate', callback)
+            widget.get_child().connect('activate', callback)
 
 
 class DateTimes(Dates):
@@ -85,7 +84,7 @@ class DateTimes(Dates):
             if isinstance(widget, Date):
                 widget.connect('activate', callback)
             elif isinstance(widget, Time):
-                widget.child.connect('activate', callback)
+                widget.get_child().connect('activate', callback)
 
 
 class Selection(gtk.ScrolledWindow):
@@ -133,23 +132,21 @@ class ScreenContainer(object):
         self.search_table = None
         self.last_search_text = ''
         self.tab_domain = tab_domain or []
+        self.tab_counter = []
 
         tooltips = common.Tooltips()
 
         self.filter_vbox = gtk.VBox(spacing=0)
         self.filter_vbox.set_border_width(0)
         hbox = gtk.HBox(homogeneous=False, spacing=0)
-        self.filter_button = gtk.ToggleButton()
-        self.filter_button.set_use_underline(True)
-        self.filter_button.set_label(_('F_ilters'))
-        self.filter_button.set_relief(gtk.RELIEF_NONE)
-        self.filter_button.set_alignment(0.0, 0.5)
-        self.filter_button.connect('toggled', self.search_box)
-        hbox.pack_start(self.filter_button, expand=False, fill=False)
 
         self.search_entry = PlaceholderEntry()
         self.search_entry.set_placeholder_text(_('Search'))
         self.search_entry.set_alignment(0.0)
+        self.search_entry.set_icon_from_stock(
+            gtk.ENTRY_ICON_PRIMARY, 'tryton-find')
+        self.search_entry.set_icon_tooltip_text(
+            gtk.ENTRY_ICON_PRIMARY, _('Open filters'))
         self.completion = gtk.EntryCompletion()
         self.completion.set_model(gtk.ListStore(str))
         self.completion.set_text_column(0)
@@ -167,20 +164,22 @@ class ScreenContainer(object):
 
         def popup(widget):
             menu = widget._menu
-            for child in menu.children():
+            for child in menu.get_children():
                 menu.remove(child)
             if not widget.props.active:
                 menu.popdown()
                 return
 
-            def menu_position(menu):
-                x, y = widget.window.get_origin()
+            def menu_position(menu, data=None):
                 widget_allocation = widget.get_allocation()
-                return (
-                    widget_allocation.x + x,
-                    widget_allocation.y + widget_allocation.height + y,
-                    False
-                )
+                if hasattr(widget.window, 'get_root_coords'):
+                    x, y = widget.window.get_root_coords(
+                        widget_allocation.x, widget_allocation.y)
+                else:
+                    x, y = widget.window.get_origin()
+                    x += widget_allocation.x
+                    y += widget_allocation.y
+                return (x, y + widget_allocation.height, False)
 
             for id_, name, domain in self.bookmarks():
                 menuitem = gtk.MenuItem(name)
@@ -243,12 +242,22 @@ class ScreenContainer(object):
 
         if self.tab_domain:
             self.notebook = gtk.Notebook()
-            self.notebook.props.homogeneous = True
+            try:
+                self.notebook.props.homogeneous = True
+            except AttributeError:
+                # No more supported by GTK+3
+                pass
             self.notebook.set_scrollable(True)
-            for name, domain in self.tab_domain:
+            for name, domain, count in self.tab_domain:
+                hbox = gtk.HBox(spacing=3)
                 label = gtk.Label('_' + name)
                 label.set_use_underline(True)
-                self.notebook.append_page(gtk.VBox(), label)
+                hbox.pack_start(label, expand=True, fill=True)
+                counter = gtk.Label()
+                hbox.pack_start(counter, expand=False, fill=True)
+                hbox.show_all()
+                self.notebook.append_page(gtk.VBox(), hbox)
+                self.tab_counter.append(counter)
             self.filter_vbox.pack_start(self.notebook, expand=True, fill=True)
             self.notebook.show_all()
             # Set the current page before connecting to switch-page to not
@@ -270,6 +279,10 @@ class ScreenContainer(object):
 
         tooltips.enable()
 
+    def destroy(self):
+        if self.search_window:
+            self.search_window.hide()
+
     def widget_get(self):
         return self.vbox
 
@@ -291,9 +304,6 @@ class ScreenContainer(object):
     def hide_filter(self):
         if self.filter_vbox:
             self.filter_vbox.hide()
-        if self.filter_button and self.filter_button.get_active():
-            self.filter_button.set_active(False)
-            self.filter_button.toggled()
         if self.notebook:
             self.notebook.set_show_tabs(False)
             if self.viewport not in self.vbox.get_children():
@@ -302,19 +312,17 @@ class ScreenContainer(object):
                 self.vbox.pack_end(self.viewport)
 
     def set(self, widget):
+        viewport1 = self.viewport
+        viewport2 = self.alternate_viewport
         if self.alternate_view:
-            if self.alternate_viewport.get_child():
-                self.alternate_viewport.remove(
-                        self.alternate_viewport.get_child())
-            if widget == self.viewport.get_child():
-                self.viewport.remove(self.viewport.get_child())
-            self.alternate_viewport.add(widget)
-            self.alternate_viewport.show_all()
-            return
-        if self.viewport.get_child():
-            self.viewport.remove(self.viewport.get_child())
-        self.viewport.add(widget)
-        self.viewport.show_all()
+            viewport1, viewport2 = viewport2, viewport1
+
+        if viewport1.get_child():
+            viewport1.remove(viewport1.get_child())
+        if widget == viewport2.get_child():
+            viewport2.remove(widget)
+        viewport1.add(widget)
+        viewport1.show_all()
 
     def update(self):
         res = self.screen.search_complete(self.get_text())
@@ -340,31 +348,32 @@ class ScreenContainer(object):
         self.do_search()
 
     def bookmark_match(self):
-        current_text = self.get_text()
-        current_domain = self.screen.domain_parser.parse(current_text)
-        self.search_entry.set_icon_activatable(gtk.ENTRY_ICON_SECONDARY,
-            bool(current_text))
-        self.search_entry.set_icon_sensitive(gtk.ENTRY_ICON_SECONDARY,
-            bool(current_text))
         icon_stock = self.search_entry.get_icon_stock(gtk.ENTRY_ICON_SECONDARY)
-        for id_, name, domain in self.bookmarks():
-            text = self.screen.domain_parser.string(domain)
-            domain = self.screen.domain_parser.parse(text.decode('utf-8'))
-            if (text == current_text
-                    or domain == current_domain):
-                if icon_stock != 'tryton-star':
-                    self.search_entry.set_icon_from_stock(
-                        gtk.ENTRY_ICON_SECONDARY, 'tryton-star')
-                self.search_entry.set_icon_tooltip_text(
-                    gtk.ENTRY_ICON_SECONDARY, _('Remove this bookmark'))
-                return id_
+        current_text = self.get_text()
+        if current_text:
+            current_domain = self.screen.domain_parser.parse(current_text)
+            self.search_entry.set_icon_activatable(gtk.ENTRY_ICON_SECONDARY,
+                bool(current_text))
+            self.search_entry.set_icon_sensitive(gtk.ENTRY_ICON_SECONDARY,
+                bool(current_text))
+            for id_, name, domain in self.bookmarks():
+                text = self.screen.domain_parser.string(domain)
+                domain = self.screen.domain_parser.parse(text.decode('utf-8'))
+                if (text == current_text
+                        or domain == current_domain):
+                    if icon_stock != 'tryton-star':
+                        self.search_entry.set_icon_from_stock(
+                            gtk.ENTRY_ICON_SECONDARY, 'tryton-star')
+                    self.search_entry.set_icon_tooltip_text(
+                        gtk.ENTRY_ICON_SECONDARY, _('Remove this bookmark'))
+                    return id_
         if icon_stock != 'tryton-unstar':
             self.search_entry.set_icon_from_stock(gtk.ENTRY_ICON_SECONDARY,
                 'tryton-unstar')
         if current_text:
             self.search_entry.set_icon_tooltip_text(gtk.ENTRY_ICON_SECONDARY,
                 _('Bookmark this filter'))
-        else:
+        elif self.search_entry.get_icon_tooltip_text(gtk.ENTRY_ICON_SECONDARY):
             self.search_entry.set_icon_tooltip_text(gtk.ENTRY_ICON_SECONDARY,
                 None)
 
@@ -384,6 +393,7 @@ class ScreenContainer(object):
     def switch_page_after(self, notebook, page, page_num):
         self.do_search()
         notebook.grab_focus()
+        self.screen.count_tab_domain()
 
     def get_tab_domain(self):
         if not self.notebook:
@@ -393,6 +403,26 @@ class ScreenContainer(object):
             return []
         return self.tab_domain[idx][1]
 
+    def set_tab_counter(self, count, idx=None):
+        if not self.tab_counter or not self.notebook:
+            return
+        if idx is None:
+            idx = self.notebook.get_current_page()
+        if idx < 0:
+            return
+        label = self.tab_counter[idx]
+        tooltip = common.Tooltips()
+        if count is None:
+            label.set_label('')
+            tooltip.set_tip(label, '')
+        else:
+            tooltip.set_tip(label, '%d' % count)
+            fmt = '(%d)'
+            if count > 99:
+                fmt = '(%d+)'
+                count = 99
+            label.set_label(fmt % count)
+
     def match_selected(self, completion, model, iter):
         def callback():
             if not self.search_entry.props.window:
@@ -421,7 +451,9 @@ class ScreenContainer(object):
         gobject.idle_add(keypress)
 
     def icon_press(self, widget, icon_pos, event):
-        if icon_pos == 1:
+        if icon_pos == gtk.ENTRY_ICON_PRIMARY:
+            self.search_box(widget)
+        elif icon_pos == gtk.ENTRY_ICON_SECONDARY:
             icon_stock = self.search_entry.get_icon_stock(icon_pos)
             model_name = self.screen.model_name
             if icon_stock == 'tryton-unstar':
@@ -445,14 +477,18 @@ class ScreenContainer(object):
         self.update()
         self.search_entry.emit('changed')
 
-    def search_box(self, button):
-        def key_press(window, event):
+    def search_box(self, widget):
+        def window_hide(window, *args):
+            window.hide()
+            self.search_entry.grab_focus()
+
+        def key_press(widget, event):
             if event.keyval == gtk.keysyms.Escape:
-                button.set_active(False)
-                window.hide()
+                window_hide(widget)
+                return True
+            return False
 
         def search():
-            button.set_active(False)
             self.search_window.hide()
             text = ''
             for label, entry in self.search_table.fields:
@@ -472,16 +508,15 @@ class ScreenContainer(object):
 
         if not self.search_window:
             self.search_window = gtk.Window()
-            self.search_window.set_transient_for(button.get_toplevel())
+            self.search_window.set_transient_for(widget.get_toplevel())
             self.search_window.set_type_hint(
                 gtk.gdk.WINDOW_TYPE_HINT_POPUP_MENU)
             self.search_window.set_destroy_with_parent(True)
-            self.search_window.set_title('Tryton')
-            self.search_window.set_icon(TRYTON_ICON)
             self.search_window.set_decorated(False)
             self.search_window.set_deletable(False)
-            self.search_window.connect('delete-event', lambda *a: True)
+            self.search_window.connect('delete-event', window_hide)
             self.search_window.connect('key-press-event', key_press)
+            self.search_window.connect('focus-out-event', window_hide)
             vbox = gtk.VBox()
             fields = [f for f in self.screen.domain_parser.fields.itervalues()
                 if f.get('searchable', True)]
@@ -553,15 +588,15 @@ class ScreenContainer(object):
                     scrolled.size_request()))
             self.search_window.set_default_size(*new_size)
 
-        parent = button.get_toplevel()
-        button_x, button_y = button.translate_coordinates(parent, 0, 0)
-        button_allocation = button.get_allocation()
+        parent = widget.get_toplevel()
+        widget_x, widget_y = widget.translate_coordinates(parent, 0, 0)
+        widget_allocation = widget.get_allocation()
 
         # Resize the window to not be out of the parent
         width, height = self.search_window.get_default_size()
         allocation = parent.get_allocation()
-        delta_width = allocation.width - (button_x + width)
-        delta_height = allocation.height - (button_y + button_allocation.height
+        delta_width = allocation.width - (widget_x + width)
+        delta_height = allocation.height - (widget_y + widget_allocation.height
             + height)
         if delta_width < 0:
             width += delta_width
@@ -570,32 +605,25 @@ class ScreenContainer(object):
         self.search_window.resize(width, height)
 
         # Move the window under the button
-        x, y = button.window.get_origin()
-        self.search_window.move(
-            x + button_allocation.x,
-            y + button_allocation.y + button_allocation.height)
-
-        from tryton.gui.main import Main
-        page = Main.get_main().get_page()
-        if button.get_active():
-            if page and self.search_window not in page.dialogs:
-                page.dialogs.append(self.search_window)
-            self.search_window.show()
-
-            if self.last_search_text.strip() != self.get_text().strip():
-                for label, entry in self.search_table.fields:
-                    if isinstance(entry, gtk.ComboBox):
-                        entry.set_active(-1)
-                    elif isinstance(entry, Dates):
-                        entry.set_values(None, None)
-                    elif isinstance(entry, Selection):
-                        entry.treeview.get_selection().unselect_all()
-                    else:
-                        entry.set_text('')
-                if self.search_table.fields:
-                    self.search_table.fields[0][1].grab_focus()
-
+        if hasattr(widget.window, 'get_root_coords'):
+            x, y = widget.window.get_root_coords(
+                widget_allocation.x, widget_allocation.y)
         else:
-            self.search_window.hide()
-            if page and self.search_window in page.dialogs:
-                page.dialogs.remove(self.search_window)
+            x, y = widget.window.get_origin()
+        self.search_window.move(
+            x, y + widget_allocation.height)
+        self.search_window.show()
+        self.search_window.grab_focus()
+
+        if self.last_search_text.strip() != self.get_text().strip():
+            for label, entry in self.search_table.fields:
+                if isinstance(entry, gtk.ComboBox):
+                    entry.set_active(-1)
+                elif isinstance(entry, Dates):
+                    entry.set_values(None, None)
+                elif isinstance(entry, Selection):
+                    entry.treeview.get_selection().unselect_all()
+                else:
+                    entry.set_text('')
+            if self.search_table.fields:
+                self.search_table.fields[0][1].grab_focus()
