@@ -77,7 +77,10 @@ class WinExport(WinCSV):
     def add_chooser(self, box):
         hbox_csv_export = gtk.HBox()
         box.pack_start(hbox_csv_export, False, True, 0)
-        self.saveas = gtk.combo_box_new_text()
+        if hasattr(gtk, 'ComboBoxText'):
+            self.saveas = gtk.ComboBoxText()
+        else:
+            self.saveas = gtk.combo_box_new_text()
         hbox_csv_export.pack_start(self.saveas, True, True, 0)
         self.saveas.append_text(_("Open"))
         self.saveas.append_text(_("Save"))
@@ -145,9 +148,10 @@ class WinExport(WinCSV):
 
     def sig_unsel(self, *args):
         store, paths = self.view2.get_selection().get_selected_rows()
-        while paths:
-            store.remove(store.get_iter(paths[0]))
-            store, paths = self.view2.get_selection().get_selected_rows()
+        # Convert first into TreeIter before removing from the store
+        iters = [store.get_iter(p) for p in paths]
+        for i in iters:
+            store.remove(i)
 
     def sig_unsel_all(self, *args):
         self.model2.clear()
@@ -181,7 +185,7 @@ class WinExport(WinCSV):
         self.pref_export.set_model(self.predef_model)
 
     def addreplace_predef(self, widget):
-        iter = self.model2.get_iter_root()
+        iter = self.model2.get_iter_first()
         fields = []
         while iter:
             field_name = self.model2.get_value(iter, 1)
@@ -275,7 +279,7 @@ class WinExport(WinCSV):
         if response == gtk.RESPONSE_OK:
             fields = []
             fields2 = []
-            iter = self.model2.get_iter_root()
+            iter = self.model2.get_iter_first()
             while iter:
                 fields.append(self.model2.get_value(iter, 1))
                 fields2.append(self.model2.get_value(iter, 0))
@@ -319,12 +323,12 @@ class WinExport(WinCSV):
                 writer.writerow(row)
             if popup:
                 if len(data) == 1:
-                    common.message(_('%d record saved!') % len(data))
+                    common.message(_('%d record saved.') % len(data))
                 else:
-                    common.message(_('%d records saved!') % len(data))
+                    common.message(_('%d records saved.') % len(data))
             return True
         except IOError, exception:
-            common.warning(_("Operation failed!\nError message:\n%s")
+            common.warning(_("Operation failed.\nError message:\n%s")
                 % exception, _('Error'))
             return False
 
